/**
 * Created by dlm on 11/08/16.
 */

function parseDateVal(item, dateKey) {

	var date = item[dateKey].split(/[\- ]/g),
		mese = parseInt(date[1]) - 1,
		mesi = 'Gennaio,Febbraio,Marzo,Aprile,Maggio,Giugno,Luglio,Agosto,Settembre,Ottobre,Novembre,Dicembre'.split(/,/g);

	item.year = date[0];
	item.month = mesi[mese];
	item.day = date[2];

	return item;
}

var Period = function(){

	this.hours = 0;
	this.days = 0;
	this.weeks = 0;
	this.months = 0;
	this.years = 0;


	var _date = new Date(),
		_format = "%YY-%MM-%DD %HH";

	this.setDate = function(d){
		_date = d;
	};

	this.setFormat = function(format){
		_format = format;
	};

	this.toString = function(){

		_date.setDate( _date.getDate()+this.days +(this.weeks*7));

		_date.setMonth( _date.getMonth()+this.months );
		_date.setYear( _date.getFullYear()+this.years );
		_date.setHours( _date.getHours()+this.hours );

		return _format
			.replace(Period.FMT_FULL_HOUR, ('0' + _date.getHours()).substr(-2))
			.replace(Period.FMT_FULL_DAY, ('0' + _date.getDate()).substr(-2))
			.replace(Period.FMT_FULL_MONTH, ('0' + _date.getMonth()).substr(-2))
			.replace(Period.FMT_FULL_YEAR, _date.getFullYear().toString())
			.replace(Period.FMT_HOUR,  _date.getHours().toString())
			.replace(Period.FMT_DAY,  _date.getDate().toString())
			.replace(Period.FMT_MONTH,  _date.getMonth().toString())
			.replace(Period.FMT_YEAR,  _date.getYear().toString().substr(-2));
	};

};

Period.FMT_FULL_DAY = '%DD';
Period.FMT_DAY = '%D';
Period.FMT_FULL_MONTH = '%MM';
Period.FMT_MONTH = '%M';
Period.FMT_YEAR = '%Y';
Period.FMT_FULL_YEAR = '%YY';
Period.FMT_HOUR = '%H';
Period.FMT_FULL_HOUR = '%HH';

function periodicalAsk(args){

	args.period = args.period || {};
	args.storeAs = args.storeAs || args.key;
	if(args.storeAs == false){
		showErrorMessage("Application error", "Invocazione invalida di periodicalAsk con parametro <code>key</code> invalido.", 'app-err-01');
		return;
	}
	args.callback = args.callback || function(){ console.error('Undefined callback for ' + args.key, arguments)};
	var p = new Period();
	p.setDate( new Date());
	p.setFormat(
		Period.FMT_FULL_YEAR+'-'+
		Period.FMT_FULL_MONTH+'-'+
		Period.FMT_FULL_DAY+' '+
		Period.FMT_FULL_HOUR);

	p.hours = 0;
	p.months = 0;
	p.days = 0;
	p.weeks = 0;
	p.years = 0;

	var d = p.toString();


	console.log(args.storeAs);
	var appData = localStorage.getItem(args.storeAs + '-cached') || ('{ "expiresOn": "'+d+'"}'),
		cachedData = JSON.parse(appData),
		ld = cachedData.expiresOn,
		online = applicationOnLine,
		latestData;
	console.log(d, ld, cachedData);
	if (d >= ld && online) {
		ask({
			url: args.url,
			method: args.method,
			data: args.arguments
		}).success(function (response) {
			if (response.status == 'OK') {

				p.hours = args.period.hours || 0;
				p.months = args.period.months || 0;
				p.days = args.period.days || 0;
				p.weeks = args.period.weeks || 0;
				p.years = args.period.years || 0;

				latestData = {
					expiresOn: p.toString(),
					items: (args.key!==false)?response[args.key]:response
				};

				localStorage.setItem(args.storeAs + '-cached', JSON.stringify(latestData));
				args.callback(latestData.items);
			}
		}).error(function (response) {
			console.log(arguments);
		});
	} else {
		try {
			var sCachedData = localStorage.getItem(args.storeAs+'-cached').toString();
			cachedData = JSON.parse(sCachedData);

			latestData = cachedData.items;
		} catch (e) {
			latestData = [];
		}
		args.callback(latestData);
	}
}

function dailyAsk(key, url, method, args, callback) {

	return periodicalAsk({
		key: key,
		url: url,
		method: method,
		arguments: args,
		callback: callback,
		period: {
			days: 1
		}
	});

}


function hourlyAsk(key, url, method, args, callback) {

	return periodicalAsk({
		key: key,
		url: url,
		method: method,
		arguments: args,
		callback: callback,
		period: {
			hours: 1
		}
	});

}

function whenExists( selector, callback ){

	if( $(selector).length == 0){
		setTimeout( function(){ whenExists(selector, callback); }, 10 );
	}else{
		log( "Ho atteso la creazione di " + selector );
		callback();
	}
}

/**
 * @param title
 * @param [message]
 * @param [code]
 */
function showMessage(title, message, code, mode){
	var data = {};
	if(typeof(title) === "string") {

		data.title = title;
		data.message = message;
		data.code = code;

	}else{
		data = title;
	}

	if(mode=='error'){
		$('#output-dialog-error-code').addClass('text-danger').removeClass('text-success');
		$('#output-dialog-title').addClass('text-danger').removeClass('text-success');
	}else{
		$('#output-dialog-error-code').addClass('text-success').removeClass('text-danger');
		$('#output-dialog-title').addClass('text-success').removeClass('text-danger');
	}

	$('#output-dialog-title').html(data.title);
	$('#output-dialog-description').html(data.message);
	$('#output-dialog-error-code').html(data.code);

	$('#output-dialog').modal();

}

function showErrorMessage(title, message, code ){
	if(APP_ANALYTICS_CODE) FlurryAgent.logError(title + ' ' + message);
	showMessage(title,message,code,'error');
}

function showSuccessMessage(title, message, code ){
	showMessage(title,message,code,'success');
}
