var XtemplateRemoteLoader = (function ($) {
	'use strict';

	var _INSTANCE = null,
		xTemplateInstance = null;

	return function XtemplateRemoteLoader(settings) {

		if(_INSTANCE !== null) return _INSTANCE;

		_INSTANCE = this;

		var log = function(something){
			if(settings.log){
				console.log(something);
			}
		};


		var that = this,
				defaultSettings = {
					debug: false,
					autoload: true,
					xtemplateInstance: false,
					on: {

						// startLoad
						// endLoad

						// (Into the queue)
						// request
						// loaded
						// parsed

						// complete (at the end of the queue)

					}
				},
				triggered = {},
				applySettings = function (settings) {
					for (var key in defaultSettings) {
						if (settings[key] === undefined) {
							settings[key] = defaultSettings[key];
						}
					}

				};
		applySettings(settings);

		var resetTriggers = function(){
			for(var trigger in triggered){
				triggered[trigger] = false;
			}
		};

		that.trigger = function (eventName, arg) {

			if (settings.on[eventName] ){

				for(var i=0; i<settings.on[eventName].length; i++){
					if(typeof(settings.on[eventName][i]) === 'function') {
						var eventCallback = settings.on[eventName][i];
						eventCallback(arg);
					}
				}

			}

			triggered[eventName] = arg;
		};

		that.off = function( eventName ){
			settings.on[eventName] = [];
			return that;
		};
		that.on = function( eventName, callback ){

			if( settings.on[eventName] === undefined ) settings.on[eventName] = [];
			settings.on[eventName].push( callback );
			if(triggered[eventName]) callback(triggered[eventName]);
			return that;
		};

		that.load = function (groupName) {

			if(groupName===undefined) groupName = '*';
			resetTriggers();

			that.trigger('startLoad', groupName);

			var
					selector = (groupName == '*') ? groupName : ('[data-xtemplate-remote-group="' + groupName + '"]'),
					queue = 0,
					runme = [];

			xTemplateInstance = ( settings.xtemplateInstance ) ? settings.xtemplateInstance :
					( new Xtemplate( settings.debug ? true : undefined ) );

			$('*[data-xtemplate-remote]').filter(selector).each(function () {
				var uniqueQueueId = Math.random();
				var resource = $(this).data('xtemplate-queue-id', uniqueQueueId).data('xtemplate-remote');
				$(this)[0].removeAttribute('data-xtemplate-remote');
				queue += 1;
				runme = [];
				that.trigger('request', {
					url:        resource,
					'current':  this,
					'inQueue':  queue,
					'id':       uniqueQueueId
				});
				xTemplateInstance.wait(true);
				$.ajax({
					method: 'GET',
					url: resource,
					context: this,
					headers: {
						'requested-from': 'xtemplate'
					}
				}).complete(function(){
					queue -= 1;

					if (queue == 0) {

						xTemplateInstance.wait(false);

						runme.forEach( function( item ){

							var
									id = item.id,
									rows = item.rows,
									self = item.self;

							if($.isArray(rows))
								rows.forEach( function(item){
									item.rand = parseInt(Math.random() * 10000);
								} );
							else
								rows.rand = parseInt(Math.random() * 10000);
							var html = xTemplateInstance.apply('#'+id, rows);
							if ($(self).data('xtemplate-remote-remove')) {
								var docFrag = document.createDocumentFragment();
								$(docFrag).html(html);
								log(html);
								$(self).parents(':first').append(docFrag);
								$(self).remove();
							} else {
								$(self).html(html);
							}

						});


						that.trigger('complete');

						log( 'don\'t wait anymore');

					}

					if (settings.debug) log(queue, xTemplateInstance.wait());

				}).success(function (data) {
					if (settings.debug) log(data);

					that.trigger('loaded', {
						'current': this,
						'inQueue': $(this).data('xtemplate-queue-id')
					});

					if (/^(.|\r?\n)*<body>/im.test(data)) {
						data = data
								.replace(/^(.|\r?\n)*<body>/im, '')
								.replace(/<\/body>(.|\r?\n)*$/im, '');
					}

					var id = $(this).data('xtemplate-remote-id');

					if (id) {

						log( 'Storing the remote template with id:' + id);

						xTemplateInstance.addTemplate(id, data, '');
						if (settings.debug) log(this);
						$(this).remove();
					} else {

						id = 'xtemplate-remote-loader-' + parseInt(Math.random()*100000);

						xTemplateInstance.addTemplate(id, data, '');
						var rows = {
							empty: true
						};
						if ($(this).data('xtemplate-rows')) {
							rows = window[$(this).data('xtemplate-rows')];
						}

						runme.push( { 'id': id, 'rows': rows, self: this} );

					}

					that.trigger('parsed', {
						'current': this,
						'inQueue': $(this).data('xtemplate-queue-id')
					});

					$(this).data('xtemplate-queue-id', '');


				})
				.error(function (data) {
					$(this).html(
							'Request to: <code>' + $(this).data('xtemplate-remote') + '</code> got:' +
							'<strong>' + data.status + '</strong> <em>' + data.statusText + '</em>');
				});
			});

			if( $('*[data-xtemplate-remote]').filter(selector).length == 0 ){
				that.trigger('complete');
			}
			return this;
		};
		$(document).ready(function () {
			if (settings.autoload) that.load('*');
		});
	};
})(jQuery);